# Please note: these functions only work properly if the color definitions have been set.
# Therefore be aware that importing these functions has that side-effect.

import numpy as np
import seaborn as sns

from apol_dataviz.colors import ColorDefinitions

hex2husl = sns.external.husl.hex_to_husl
husl2hex = sns.external.husl.husl_to_hex

cd = ColorDefinitions()
cd.apply_definitions()


def apol_teal_pal(N=4, reverse=False):

    """
    apol_teal_pal

    Function that gets a sequential palette in blueTeal

    KEYWORDS:
        N = 4 - return 4 sequential colors unless specified
        reverse = False - light to dark unless true

    OUTPUTS:
        palette - list of RGBA tuples

    """

    pale = cd.cget("blueTealComplement")
    dark = cd.cget("blueTeal")

    pair = [pale, dark] if not reverse else [dark, pale]
    return sns.palettes.blend_palette(pair, N)


def apol_navy_pal(N=4, reverse=False):

    """
    apol_navy_pal

    Function that gets a sequential palette in navyBlue

    KEYWORDS:
        N = 4 - return 4 sequential colors unless specified
        reverse = False - light to dark unless true

    OUTPUTS:
        palette - list of RGBA tuples

    """

    pale = cd.cget("navyBlueComplement")
    dark = cd.cget("navyBlue")

    pair = [pale, dark] if not reverse else [dark, pale]
    return sns.palettes.blend_palette(pair, N)


def spaced_hue_palette(n_col=5, base_color=cd.blueTeal):

    """
    spaced_hue_palette

    The HuSL system is a neat way of creating color palettes that have a
    uniform apparent brightness. This function returns such a palette
    that includes one chosen color. The colors are evenly spaced in
    hue by default.

    KEYWORDS:
        n_col = 5 - number of colors to have in the palette
        base_color = cd.blueTeal - starting point for palette generation

    OUTPUTS:
        hexcodes - list of color hexcodes comprising a palette

    """

    # convert base_color hexcode -> HuSL representation
    H, S, L = hex2husl(base_color)

    hues = np.linspace(0, 360, n_col, endpoint=False)

    hues += H

    hexcode_generator = map(lambda h: husl2hex(h, S, L), hues)

    hexcodes = [hexcode for hexcode in hexcode_generator]

    return hexcodes


def spaced_saturation_palette(n_col=5, base_color=cd.blueTeal):

    """
    spaced_saturation_palette

    Returns a list (palette) of colors evenly spaced in saturation,
    interpolating through a base color

    KEYWORDS:
        n_col = 5 - number of colors to have in the palette
        base_color = cd.blueTeal - starting point for palette generation

    OUTPUTS:
        hexcodes - list of color hexcodes comprising a palette

    """

    # convert base_color hexcode -> HuSL representation
    H, S, L = hex2husl(base_color)

    sats = np.linspace(0, 100, n_col)

    hexcode_generator = map(lambda s: husl2hex(H, s, L), sats)

    hexcodes = [hexcode for hexcode in hexcode_generator]

    return hexcodes


def spaced_lightness_palette(n_col=5, base_color=cd.blueTeal):

    """
    spaced_lightness_palette

    Returns a list (palette) of colors evenly spaced in lightness,
    interpolating through a base color

    KEYWORDS:
        n_col = 5 - number of colors to have in the palette
        base_color = cd.blueTeal - starting point for palette generation

    OUTPUTS:
        hexcodes - list of color hexcodes comprising a palette

    """

    # convert base_color hexcode -> HuSL representation
    H, S, L = hex2husl(base_color)

    lis = np.linspace(0, 100, n_col)

    hexcode_generator = map(lambda l: husl2hex(H, S, l), lis)

    hexcodes = [hexcode for hexcode in hexcode_generator]

    return hexcodes


def spaced_brightness_palette(n_col=5, base_color=cd.blueTeal):

    """
    spaced_brightness_palette

    Returns a list (palette) of colors evenly spaced in lightness
    and saturation, interpolating through a base color

    KEYWORDS:
        n_col = 5 - number of colors to have in the palette
        base_color = cd.blueTeal - starting point for palette generation

    OUTPUTS:
        hexcodes - list of color hexcodes comprising a palette

    """

    # convert base_color hexcode -> HuSL representation
    H, S, L = hex2husl(base_color)

    lisats = np.linspace(0, 100, n_col)

    hexcode_generator = map(lambda x: husl2hex(H, x, x), lisats)

    hexcodes = [hexcode for hexcode in hexcode_generator]

    return hexcodes


def opposing_hue_palette(n_col=5, base_color=cd.blueTeal):

    """
    opposing_hue_palette

    Returns a diverging color palette with a chosen base-color on one
    end, its opposing, equal-apparent-brightness hue on the other (via
    the HuSL system), interpolating through white in the middle. This
    will always be the central hue of the palette if the number of
    colors in the palette is odd.

    KEYWORDS:
        n_col = 5 - number of colors to have in the palette
        base_color = cd.blueTeal - starting point for palette generation

    OUTPUTS:
        hexcodes - list of color hexcodes comprising a palette

    """

    # convert base_color hexcode -> HuSL representation
    H, S, L = hex2husl(base_color)

    H2 = H + 180

    colors_per_arm = n_col // 2

    require_midpoint = bool(n_col % 2)  # do we need a midpoint?

    saturations = np.linspace(S, 100, colors_per_arm + 1)[:-1]
    lightnesses = np.linspace(L, 100, colors_per_arm + 1)[:-1]

    arm1 = zip([H] * colors_per_arm, saturations, lightnesses)
    arm2 = zip([H2] * colors_per_arm, saturations[::-1], lightnesses[::-1])

    arm1hex = [husl2hex(h, s, l) for h, s, l in arm1]
    arm2hex = [husl2hex(h, s, l) for h, s, l in arm2]

    if not require_midpoint:
        return arm1hex + arm2hex

    midhex = [husl2hex(H, 100, 100)]

    return arm1hex + midhex + arm2hex



import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt

plt.ion()
import seaborn as sns

sns.set_color_codes()

from apol_dataviz.colab import download_lato_font
from apol_dataviz.colors import ColorDefinitions


def use_apol_style():

    """
    use_apol_style

    This function sets up the Apolitical House Style for plotting

    """

    download_lato_font()  # takes no action if not running in a Google Colab notebook

    cd = ColorDefinitions()
    cd.apply_definitions()  # register colors and colormaps with matplotlib

    apol_labelsize = 16
    apol_titlesize = 18

    ## set up the defaults for plotting

    mpl.font_manager._rebuild()

    APOLITICAL_HOUSE_STYLE = {
        "axes.axisbelow": True,
        "axes.edgecolor": cd.chartBordersRGBA,
        "axes.facecolor": "white",
        "axes.grid": True,
        "axes.labelcolor": cd.chartLabels,
        "axes.labelweight": "bold",
        "axes.labelsize": apol_labelsize,
        "axes.prop_cycle": mpl.cycler(color=cd.apol_paletteRGB),
        "axes.spines.bottom": True,
        "axes.spines.left": True,
        "axes.spines.top": False,
        "axes.spines.right": False,
        "axes.titlesize": apol_titlesize,
        "axes.titleweight": "bold",
        "figure.edgecolor": cd.chartBordersRGBA,
        "figure.facecolor": "white",
        "figure.figsize": [10, 6],
        "font.sans-serif": ["Lato"],
        "font.weight": 100,
        "grid.alpha": 0.08,
        "grid.color": cd.chartDemarcation,
        "image.cmap": "apol",
        "lines.color": cd.chartBordersRGBA,
        "patch.edgecolor": cd.chartDemarcation,
        "text.color": cd.chartLabels,
        "xtick.bottom": False,
        "xtick.labelsize": apol_labelsize,
        "xtick.top": False,
        "ytick.labelsize": apol_labelsize,
        "ytick.left": False,
        "ytick.right": False,
    }

    plt.style.use("default")
    plt.style.use(APOLITICAL_HOUSE_STYLE)


if __name__ == "__main__":
    # if style.py is directly executed, set all the definitions
    use_apol_style()
    print("Now using the Apolitical House Style for matplotlib graphics.")



import os
import pkg_resources
import sys

from importlib import resources
from zipfile import ZipFile


def download_lato_font(verbose=False):

    RUNNING_IN_COLAB = "google.colab" in sys.modules

    if not RUNNING_IN_COLAB:
        if verbose:
            print(
                """
                Not running in Google Colab. For best results ensure you have
                the Lato font installed on your system.
                """
            )
        return

    # if we are running in Google Colab, we need to install the correct font
    # from https://www.1001freefonts.com/d/5722/lato.zip to the right place:
    mpl_fonts = "/usr/local/lib/python3.6/dist-packages/matplotlib/mpl-data/fonts/ttf/"

    if verbose:
        print("... extracting lato.zip contents to mpl-data/fonts ...")

    font_file = pkg_resources.resource_filename(__name__, "fonts/lato.zip")

    with ZipFile(font_file, "r") as zip:
        zip.extractall(mpl_fonts)

    if verbose:
        print("... cleaning up ...")

    os.remove(mpl_fonts + "OFL.txt")

    if verbose:
        print("... complete.")



# colors.py - color definitions for style

import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt

plt.ion()
import seaborn as sns

sns.set_color_codes()


class ColorDefinitions:

    blueTeal = "#00B3BF"
    shockPink = "#ED1F7A"
    strongPurple = "#6B3ECD"
    paleOrange = "#FF7550"
    mintGreen = "#5ECE8A"
    goldenYellow = "#FCCA4C"
    plotGrey = "#B7C5D5"
    navyBlue = "#081D48"
    brightBlue = "#1772B8"
    strongRed = "#DC3C43"
    greenTeal = "#46B8A6"
    lightPurple = "#B75BB4"

    whiteTeal = "#EBF6F7"
    chartLabels = "#565A5C"
    chartDemarcation = "#081D48"  # 0.2 alpha for borders, 0.08 for gridlines

    apol_palette = [
        blueTeal,
        navyBlue,
        shockPink,
        plotGrey,
        strongPurple,
        paleOrange,
        mintGreen,
        goldenYellow,
        brightBlue,
        strongRed,
        greenTeal,
        lightPurple,
    ]

    apol_color_lookup = {
        "blueTeal": "#00B3BF",
        "navyBlue": "#081D48",
        "shockPink": "#ED1F7A",
        "plotGrey": "#B7C5D5",
        "strongPurple": "#6B3ECD",
        "paleOrange": "#FF7550",
        "mintGreen": "#5ECE8A",
        "goldenYellow": "#FCCA4C",
        "brightBlue": "#1772B8",
        "strongRed": "#DC3C43",
        "greenTeal": "#46B8A6",
        "lightPurple": "#B75BB4",
        "whiteTeal": "#EBF6F7",
        "blueTealComplement": "#D3EBEE",
        "navyBlueComplement": "#E8E9ED",
        "chartLabels": "#565A5C",
        "chartDemarcation": "#081D48",
    }

    def __init__(self):

        """
        __init__ for ColorDefinitions

        Adds matplotlib aliases for colors, registers colormaps

        """

        # convert the palette into alternative RGB format
        self.make_rgb_palette()
        # chart borders should have transparency:
        self.chartBordersRGBA = sns.external.husl.hex_to_rgb(
            self.cget("chartDemarcation")
        )
        # set the transparency to 0.80 (alpha=0.20)
        self.chartBordersRGBA.append(0.20)

    def apply_definitions(self):

        """
        apply_definitions

        This method wraps calls to .generate_aliases and .generate_cmaps.
        Calling it will update the matplotlib definitions to use the
        colors and palettes associated with this class

        """

        # set up matplotlib aliases for our colors
        self.generate_aliases()
        # create colormaps and register them with matplotlib
        self.generate_cmaps()

    def show_palette(self):

        """
        show_palette

        Invokes sns.palplot on self.apol_palette

        """

        sns.show_palette(self.apol_palette)

    @staticmethod
    def add_color_alias(cname, hexcode):

        """
        _add_color_alias

        This static method adds an alias for custom colors in
        matplotlib plotting methods via a string designation

        INPUTS:
            cname - name for the color
            hexcode - definition for the color

        """

        rgb = mpl.colors.colorConverter.to_rgb(hexcode)

        mpl.colors.colorConverter.colors[cname] = rgb
        mpl.colors.colorConverter.cache[cname] = rgb

    @staticmethod
    def cmap_between(hexcode1, hexcode2):

        """
        cmap_between

        This static method returns a ListedColormap between two extremal values

        INPUTS:
            hexcode1, hexcode2 - two names of known

        OUTPUTS:
            cmap - ListedColormap

        """

        gradientRGB1 = sns.external.husl.hex_to_rgb(hexcode1)
        gradientRGB2 = sns.external.husl.hex_to_rgb(hexcode2)

        n_seg = 256
        cmap_gradient = np.ones((n_seg, 4))

        for i, c in enumerate("RGB"):
            cmap_gradient[:, i] = np.linspace(gradientRGB1[i], gradientRGB2[i], n_seg)

        return mpl.colors.ListedColormap(cmap_gradient)

    def cget(self, color_name):

        """
        cget

        Convenience method: looks up a color in the definitions dict and
        returns the appropriate hexcode

        """

        return self.apol_color_lookup.get(color_name)

    def generate_aliases(self):

        """
        generate_aliases

        This method registers all colors in .apol_color_lookup with
        matplotlib so that they can be used in plotting methods directly

        """

        for cname, hexcode in self.apol_color_lookup.items():
            self.add_color_alias(cname, hexcode)

    def generate_cmaps(self):

        """
        generate_cmaps

        This method generates two color maps and registers them under

            apol (whiteTeal <--> blueTeal)
            apolBarGrad (greenTeal <--> blueTeal)

        The ListedColormap objects themselves are accessible at

            .apolCM
            .barGradientCM

        """

        self.apolCM = self.cmap_between(self.cget("whiteTeal"), self.cget("blueTeal"))

        self.barGradientCM = self.cmap_between(
            self.cget("greenTeal"), self.cget("blueTeal")
        )

        # register these colormaps
        mpl.cm.register_cmap(name="apol", cmap=self.apolCM)
        mpl.cm.register_cmap(name="apolBarGrad", cmap=self.barGradientCM)

    def make_rgb_palette(self):

        """
        make_rgb_palette

        This method sets the attribute .apol_paletteRGB, which is created
        from .apol_palette, a list of hexcodes. Sometimes the RGB format
        is easier to work with

        """

        self.apol_paletteRGB = [
            sns.external.husl.hex_to_rgb(hexcode) for hexcode in self.apol_palette
        ]



def get_ts_tick_labels(df):

    """
    get_tick_labels

    Function for producing cleanly-formatted tick labels for DataFrames
    with a time-series index

    INPUTS:
        df - DataFrame with datetime Index from which to extract
             tick labels for plotting

    OUTPUTS:
        tick_labs - array of tick labels (one per day, minimally complex)

    """

    tick_labs = df.index.strftime("%d").values

    is_first_week_of_month = df.index.day <= 7
    is_first_week_of_year = df.index.dayofyear <= 7

    tick_labs[is_first_week_of_month] = df.index[is_first_week_of_month].strftime(
        "%d\n%b"
    )

    tick_labs[is_first_week_of_year] = df.index[is_first_week_of_year].strftime(
        "%d\n%b\n%Y"
    )

    return tick_labs


def get_ts_tick_labels_monthly(df):

    """
    get_ts_tick_labels_monthly

    Function for producing cleanly-formatted tick labels for DataFrames
    with a time-series index, one per week

    INPUTS:
        df - DataFrame with datetime Index from which to extract
             tick labels for plotting

    OUTPUTS:
        tick_labs - array of tick labels (one per month, minimally complex)

    """

    tick_labs = df.index.strftime("").values

    is_first_week_of_month = df.index.day <= 7
    is_first_week_of_year = df.index.dayofyear <= 7

    tick_labs[is_first_week_of_month] = df.index[is_first_week_of_month].strftime("%b")

    tick_labs[is_first_week_of_year] = df.index[is_first_week_of_year].strftime(
        "%b\n%Y"
    )

    return tick_labs



# Functions for making a variety of specific types of plot.
# Please note: these functions only work properly if the house style has been set.
# Therefore be aware that importing these functions has that side-effect, i.e. the
# Apolitical house style will be set upon import.

from apol_dataviz.style import use_apol_style

use_apol_style()

from apol_dataviz.colors import ColorDefinitions

color_definitions = ColorDefinitions()

import numpy as np
import matplotlib.pyplot as plt

plt.ion()
import seaborn as sns


def doughnut(data, kind="categorical", sortvals=True, hole_radius=0.6):

    """
    doughnut

    Makes a doughnut plot (better than a bar chart). Various different
    flavours are available.

    INPUTS:
        data - pandas Series

    KEYWORDS:
        kind = "categorical" - style; one of {"categorical", "sequential", "plain"}
        sortvals = True      - sort the values before plotting
        hole_radius = 0.6    - change the look of the doughnut chart

    OUTPUTS:
        axis - matplotlib axis object

    """

    if kind == "categorical":
        plotargs = {"labeldistance": 1.2}

    elif kind == "sequential":
        # build a sequential palette, but soften the color differences by one unit
        plotargs = {
            "labeldistance": 1.2,
            "colors": color_definitions.apol_navy_pal(data.size + 1, reverse=True)[:-1],
        }

    else:
        plotargs = {
            "labeldistance": 1.2,
            "colors": [color_definitions.blueTeal],
            "wedgeprops": {"linewidth": 3, "edgecolor": "white"},
        }

    # if sortvals == True, sort the data before plotting
    plot_data = data.sort_values(ascending=False) if sortvals else data

    # create a piechart
    axis = plot_data.plot.pie(**plotargs)

    # add the central hole to turn it into a doughnut
    axis.add_artist(plt.Circle((0, 0), hole_radius, color="white"))

    return axis


def hbarplot(data, categorical=False):

    """
    hbarplot

    This function creates a horizontal bar plot. If the bars represent
    categories, the default palette is used. If they do not, then you can
    instead plot all bars the same color, with a nice gradient fill.

    INPUTS:
        data - pandas Series for plotting

    KEYWORDS:
        categorical = False - if True, plot bars using to default palette

    OUTPUTS:
        axis - matplotlib axis object

    """

    # note the reversal of the data; fixes unintuitive ordering of plot.barh
    axis = data.iloc[::-1].plot.barh()

    if categorical:
        axis.grid(axis="y")
        return axis

    bars = axis.patches

    grad = np.atleast_2d(np.linspace(0, 1, 256))

    xlim = axis.get_xlim()
    ylim = axis.get_ylim()

    for bar in bars:
        bar.set_facecolor("none")
        x, y = bar.get_xy()
        w, h = bar.get_width(), bar.get_height()
        axis.imshow(
            grad,
            cmap="apolBarGrad",
            extent=[x, x + w, y, y + h],
            aspect="auto",
            zorder=1,
        )

    axis.set_xlim(xlim)
    axis.set_ylim(ylim)

    axis.grid(axis="y")

    return axis


def hbarplot_round(plotdata, categorical=False):

    """
    hbarplot_round

    As with hbarplot, except the bars are bevelled. If the bars represent
    categories, the default palette is used. If they do not, then you can
    instead plot all bars the same color, with a nice gradient fill.

    INPUTS:
        data - pandas Series for plotting

    KEYWORDS:
        categorical = False - if True, plot bars using to default palette

    OUTPUTS:
        axis - matplotlib axis object
    """

    data = plotdata.iloc[::-1]  # reverse order

    fig, axis = plt.subplots()

    # we want the bar to have a normal aspect ratio, otherwise the rounding fails
    h = 10 ** np.floor(np.log10(data.max()))

    # base the rounding size on bar aspect ratio
    rounding_size = (h * data.max() * 5 / data.size) ** 0.5 / 16

    padding = 0.3
    clipbox_width = 1.1 * data.max() + padding

    grad = np.atleast_2d(np.linspace(0, 1, 256))

    counter = 0

    yticks = []
    yticknames = []

    for barname, value in data.iteritems():

        w = value
        y = counter * 2 * h

        bar_extent = [0, w, y, y + h]

        yticks.append(counter * 2 * h + h / 2)
        yticknames.append(barname)
        counter += 1

        # create the underlying bar
        if categorical:
            next_color = np.array([[apolPaletteRGB[counter]]])
            im = axis.imshow(next_color, extent=bar_extent, aspect="auto", zorder=1)
        else:
            im = axis.imshow(
                grad, cmap="apolBarGrad", extent=bar_extent, aspect="auto", zorder=1
            )

        # round the edges by clipping them
        boxstyle = mpl.patches.BoxStyle(
            "round", pad=padding, rounding_size=rounding_size
        )

        fancybox = mpl.patches.FancyBboxPatch(
            (-clipbox_width + w - 2 * padding, y + padding),
            width=clipbox_width,
            height=h - 2 * padding,
            transform=axis.transData,
            boxstyle=boxstyle,
        )

        im.set_clip_path(fancybox)

    axis.set_yticks(yticks)
    axis.set_yticklabels(yticknames)

    axis.set_xlim(0, data.max() * 1.1)
    axis.set_ylim(-h / 2, counter * 2 * h)

    axis.grid(axis="y")

    return axis


def heatmap(data):

    """
    heatmap

    Convenience function, thin wrapper around sns.heatmap that sets
    cmap to "apol" and tick rotations to 45 (x) and 0 (y). If you
    don't want these defaults then use sns.heatmap directly.

    INPUTS:
        data - pandas DataFrame

    OUTPUTS:
        axis - matplotlib axis object

    """

    axis = sns.heatmap(data, cmap="apol")

    axis.tick_params(axis="x", rotation=45)
    axis.tick_params(axis="y", rotation=0)

    return axis
